# The "Postpone text" plugin for Tkabber.
#
# Provides a private hidden text buffer for each chat input window
# and a binding to operate with it.
# The idea is to provide for quick moving of the text typed into the chat
# input window to that buffer, and then moving it back to the input window
# with the same event (say, a keystroke).
# This is helpful when the user types in some elaborate text and realizes she
# wants to quickly post another text to the same chat and then continue with
# editing.
# The command event operates depending on whether the buffer is full or not:
# when it's empty, the command event moves the text to the buffer, when it's
# full, the command event moves the text back to the input window.
#
# The command event is <<ChatPostponeText>>, the default binding to it
# is maintained via the Customize mechanism. You can disable the default
# binding by setting it's event specification to the empty string.
#
# Written by Konstantin Khomoutov <flatworm at users dot sourceforge dot com>
#
# $Id: postpone.tcl 59 2006-11-13 18:50:08Z kostix $



namespace eval postpone {
	variable options
	variable state
	variable event <<ChatPostponeText>>
	variable prevbinding ""

	custom::defgroup Plugins [::msgcat::mc "Plugins options."] -group Tkabber

	set group {Postpone Text}

	custom::defgroup $group \
		[::msgcat::mc "Text postponing buffer provides for moving the text\
			currently present in the chat input window to the special hidden\
			buffer (private to that input window)\
			and then get it back at any time."] \
		-group Chat

	custom::defvar options(binding) <Control-q> \
		[::msgcat::mc "A Tk event in the chat input window to bind the\
			push/pop operations with the postpone buffer to.\
			Set this to the empty string to remove the default binding."] \
		-group $group -type string \
		-command [namespace current]::on_binding_changed

	custom::defvar options(paste_mode) atend \
		[::msgcat::mc "How pasting from the postpone buffer is handled."] \
		-group $group -type options -values [list \
			destructive [::msgcat::mc "Destructive (replace)"] \
			atcursor    [::msgcat::mc "At the cursor position"] \
			atstart     [::msgcat::mc "At the beginning"] \
			atend       [::msgcat::mc "At the end"] \
		]

	custom::defvar options(active_background) default \
		[::msgcat::mc "Set input window background to this color when\
			the postpone buffer is holding some text."] \
		-group $group -type string

	unset group

	::hook::add open_chat_post_hook [namespace current]::setup_bindings
}



proc postpone::configure_text_widget {w} {
	variable options
	variable state

	set state($w,buffer) ""

	bind $w <Destroy> +[list \
		[namespace current]::cleanup_text_widget $w %W]

	bind $w <<ChatPostponeText>> [list \
		[namespace current]::buffer_push_pop $w]
	bind $w <<ChatPostponeText>> +break
}



proc postpone::cleanup_text_widget {w1 w2} {
	variable state

	if {$w1 != $w2} return

	array unset state $w1,*
}



proc postpone::buffer_push_pop {w} {
	variable state

	if {$state($w,buffer) == ""} {
		buffer_push $w
	} else {
		buffer_pop $w
	}
}



proc postpone::buffer_push {w} {
	variable options
	variable state

	if {[$w compare 1.0 == {end - 1 char}]} return ;# empty

	set state($w,buffer) [$w get 1.0 {end - 1 char}] ;# don't get last newline

	$w delete 1.0 end

	if {$options(active_background) != "default"} {
		set state($w,background) [$w cget -background]

		$w configure -background $options(active_background)
	}
}



proc postpone::buffer_pop {w} {
	variable options
	variable state

	switch -- $options(paste_mode) {
		destructive {
			$w delete 1.0 end
			set at end
		}
		atcursor {
			set at insert
		}
		atstart {
			set at 1.0
		}
		atend {
			set at end
		}
		default {
			error "Unknown paste_mode: $options(paste_mode)"
		}
	}

	$w insert $at $state($w,buffer)

	set state($w,buffer) ""

	if {$options(active_background) != "default"} {
		$w configure -background $state($w,background)
	}
}



proc postpone::setup_bindings {chatid type} {
	configure_text_widget [::chat::input_win $chatid]
}



# Write trace firing when "invoke postpone buffer command" event option
# is changed. It tries to bind something to the user-supplied event to
# see whether this event is valid. If it is not, it restores the old value
# of the event.
# TODO good candidate to go outside.
proc postpone::on_binding_changed {args} {
	variable options
	variable event
	variable prevbinding

	if {$options(binding) == $prevbinding} return ;# no change

	if {$options(binding) == ""} { # empty value => remove binding
		event delete $event $prevbinding
		set prevbinding ""
		return
	}

	set f [frame .testframemustnotexist]

	if {[catch {bind $f $options(binding) bogus}]} {
		global errorInfo
		set errorInfo [::msgcat::mc "...testing event valididy..."]
		bgerror [::msgcat::mc "Bad event: $options(binding)"]
		set options(binding) $prevbinding
	} else {
		event delete $event $prevbinding
		event add $event $options(binding)
		set prevbinding $options(binding)
	}

	destroy $f
}



namespace eval postpone {
	on_binding_changed ;# apply initial binding
}

